# Generated by roxygen2: do not edit by hand

export(ViewR)
export(install_viewr_deps)
export(renderViewdt)
export(save_viewdt)
export(viewdt)
export(viewdtOutput)
export(viewdt_options)
importFrom(DT,dataTableOutput)
importFrom(DT,datatable)
importFrom(DT,formatStyle)
importFrom(DT,renderDataTable)
importFrom(DT,styleColorBar)
importFrom(htmltools,htmlEscape)
importFrom(htmltools,tags)
importFrom(htmltools,withTags)
importFrom(jsonlite,toJSON)
importFrom(rhandsontable,hot_table)
importFrom(rhandsontable,hot_to_r)
importFrom(rhandsontable,rHandsontableOutput)
importFrom(rhandsontable,renderRHandsontable)
importFrom(rhandsontable,rhandsontable)
importFrom(shiny,HTML)
importFrom(shiny,actionButton)
importFrom(shiny,br)
importFrom(shiny,browserViewer)
importFrom(shiny,checkboxGroupInput)
importFrom(shiny,checkboxInput)
importFrom(shiny,column)
importFrom(shiny,conditionalPanel)
importFrom(shiny,dialogViewer)
importFrom(shiny,div)
importFrom(shiny,fluidPage)
importFrom(shiny,fluidRow)
importFrom(shiny,h5)
importFrom(shiny,icon)
importFrom(shiny,insertUI)
importFrom(shiny,isolate)
importFrom(shiny,observe)
importFrom(shiny,observeEvent)
importFrom(shiny,outputOptions)
importFrom(shiny,paneViewer)
importFrom(shiny,reactive)
importFrom(shiny,reactiveValues)
importFrom(shiny,removeUI)
importFrom(shiny,renderText)
importFrom(shiny,req)
importFrom(shiny,runGadget)
importFrom(shiny,selectInput)
importFrom(shiny,shinyApp)
importFrom(shiny,showNotification)
importFrom(shiny,sliderInput)
importFrom(shiny,stopApp)
importFrom(shiny,tabPanel)
importFrom(shiny,tabsetPanel)
importFrom(shiny,tagList)
importFrom(shiny,tags)
importFrom(shiny,textInput)
importFrom(shiny,updateCheckboxGroupInput)
importFrom(shiny,verbatimTextOutput)
importFrom(shiny,wellPanel)
importFrom(shinyjs,runjs)
importFrom(shinyjs,useShinyjs)
importFrom(shinythemes,shinytheme)
importFrom(stats,setNames)
importFrom(utils,head)
