#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom shiny actionButton checkboxGroupInput checkboxInput column
#'   conditionalPanel debounce div downloadButton downloadHandler fluidPage
#'   fluidRow h5 icon insertUI isolate observe observeEvent outputOptions
#'   plotOutput radioButtons reactive reactiveValues removeUI renderPlot
#'   renderPrint renderText runGadget selectInput shinyApp showNotification
#'   sliderInput stopApp tabPanel tabsetPanel tagList tags textInput
#'   updateCheckboxGroupInput updateSelectInput updateSliderInput wellPanel
#'   browserViewer dialogViewer paneViewer br verbatimTextOutput req HTML
#' @importFrom DT datatable dataTableOutput renderDataTable formatStyle
#'   styleColorBar
#' @importFrom rhandsontable rhandsontable rHandsontableOutput
#'   renderRHandsontable hot_to_r hot_table
#' @importFrom shinyjs useShinyjs runjs
#' @importFrom shinythemes shinytheme
#' @importFrom htmltools htmlEscape withTags tags
#' @importFrom jsonlite toJSON
#' @importFrom graphics barplot boxplot hist par plot.new rug text
#' @importFrom stats setNames
#' @importFrom utils head write.csv
## usethis namespace: end
NULL
